/******************************************************************************
 * commandedit.cpp - User edited a field in the tree
 *
 * yamce - Yet Another Minecraft Editor
 * Copyright(c) 2019 - Dick Balaska, and BuckoSoft.
 *
 * yamce is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * yamce is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *****************************************************************************/
#include <QDebug>

#include "nbtobject.h"
#include "treemodel.h"
#include "treeitem.h"
#include "commandman.h"
#include "resourcedock.h"
#include "commandedit.h"

CommandEdit::CommandEdit()
{
}

void CommandEdit::setEditData(QModelIndex modelIndex, QVariant data)
{
	m_modelIndex = modelIndex;
	m_data = data;
	TreeItem* item = static_cast<TreeItem*>(modelIndex.internalPointer());
	QString name = item->getPath();
	setName(QString("Edit %1").arg(name));
}

void  CommandEdit::doCommand()
{
	if (DEBUGCOMMAND)
		qDebug() << "CommandEdit:doCommand";
	TreeModel* tm = m_group->getTreeModel();

	TreeItem* pi = static_cast<TreeItem*>(m_modelIndex.internalPointer());
	NBTObject* obj = pi->getObject();
	m_object = *obj;

	tm->setData(m_modelIndex, m_data, Qt::DisplayRole);
	if (m_modelIndex.column() == tmName) {
		obj->name = m_data.toString();
	} else if (m_modelIndex.column() == tmData) {
		obj->setValue(m_data);
	}
	m_file = m_commandMan->getFile(pi);

}

void  CommandEdit::undoCommand()
{
	TreeModel* tm = m_group->getTreeModel();
	TreeItem* pi = static_cast<TreeItem*>(m_modelIndex.internalPointer());
	NBTObject* obj = pi->getObject();
	if (m_modelIndex.column() == tmName) {
		obj->name = m_object.name;
	} else if (m_modelIndex.column() == tmData) {
		obj->setValue(m_object.getValue());
		tm->setData(m_modelIndex, m_object.getValue(), Qt::DisplayRole);
	}
}

void  CommandEdit::redoCommand()
{
	TreeModel* tm = m_group->getTreeModel();

	TreeItem* pi = static_cast<TreeItem*>(m_modelIndex.internalPointer());
	NBTObject* obj = pi->getObject();

	tm->setData(m_modelIndex, m_data, Qt::DisplayRole);
	if (m_modelIndex.column() == tmName) {
		obj->name = m_data.toString();
	} else if (m_modelIndex.column() == tmData) {
		obj->setValue(m_data);
	}

}
