/******************************************************************************
 * commandman.h - Command Manager (the editor)
 *
 * yamce - Yet Another Minecraft Editor
 * Copyright(c) 2019 - Dick Balaska, and BuckoSoft.
 *
 * yamce is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * yamce is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *****************************************************************************/
#ifndef COMMANDMAN_H
#define COMMANDMAN_H

#include <QObject>
#include <QAction>
#include <QList>

class Command;
class MainWindow;
class NBTFile;
class ResourceDock;
class TreeItem;

class CommandMan : public QObject
{
	Q_OBJECT
public:
	CommandMan(MainWindow* mainWindow);
	~CommandMan();

	void	setUndoRedo(QAction* undo, QAction* redo, QAction* save, QAction* sync);

	ResourceDock*	getResourceDock();
	NBTFile*		getFile(TreeItem* treeItem);
	MainWindow*		getMainWindow();
	void			worldSaved();

public slots:
	void	doCommand(Command* command);
	void	undo();
	void	redo();


	/// If there are any commands for this file in the list, then mark him dirty in the TreeView and the file
	void	markDirtyOrClean(NBTFile* file);

private:
	void	postCommand(Command* c);	// after command, enable menus, mark dirty, etc.
	void	enableUndoRedo();

	MainWindow*		m_mainWindow;
	QList<Command*>	m_undoList;
	QList<Command*>	m_redoList;
	//int				m_commandIndex;
	QAction*		m_undoAction;
	QAction*		m_redoAction;
	QAction*		m_saveAction;
	QAction*		m_syncAction;

};

inline MainWindow* CommandMan::getMainWindow() { return(m_mainWindow); }

#endif // COMMANDMAN_H
