/******************************************************************************
 * commandrailmap.cpp - Apply the rail data to this map
 *
 * yamce - Yet Another Minecraft Editor
 * Copyright(c) 2019 - Dick Balaska, and BuckoSoft.
 *
 * yamce is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * yamce is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *****************************************************************************/
#include <QDebug>
#include <QPoint>
#include "nbtobject.h"
#include "nbtfile.h"
#include "commandrailmap.h"

CommandRailMap::CommandRailMap()
	: m_points(nullptr),
	  m_mapColor(18)
{
}

CommandRailMap::~CommandRailMap()
{
	if (m_points)
		delete m_points;
}

void CommandRailMap::setFile(NBTFile* file) {
	m_file = file;
	setName(QString("Rail %1").arg(m_file->getFileName()));
}

void CommandRailMap::doCommand()
{
	char red[2] = {18, 0};
	int penWidth = 2;
	int plus = penWidth/2;			// draw from minus to plus in the x and z directions
	int minus = penWidth-plus-1;
	NBTObject*	data = m_file->findObject("*:data:colors");
	if (!data) {
		qCritical() << "MapItem:load failed to find data:colors";
		return;
	}
	red[0] = m_mapColor;
	for (QPoint point : *m_points) {
		for (int w=point.x()+minus; w<point.x()+plus; w++) {
			if (w < 0 || w >=128)
				continue;
			for (int h=point.y()+minus; h<point.y()+plus; h++) {
				if (h < 0 || h >= 128)
					continue;
				m_originalData.append(data->d_byte_array->at(h*128+w));
				data->d_byte_array->replace(h*128+w, 1, &red[0], 1);

			}
		}
	}
}

void CommandRailMap::undoCommand()
{
	int penWidth = 2;
	int plus = penWidth/2;			// draw from minus to plus in the x and z directions
	int minus = penWidth-plus-1;
	char color[2] = {0, 0};
	NBTObject*	data = m_file->findObject("*:data:colors");
	if (!data) {
		qCritical() << "MapItem:load failed to find data:colors";
		return;
	}
	int i = 0;
	for (QPoint point : *m_points) {
		for (int w=point.x()+minus; w<point.x()+plus; w++) {
			if (w < 0 || w >=128)
				continue;
			for (int h=point.y()+minus; h<point.y()+plus; h++) {
				if (h < 0 || h >= 128)
					continue;
				color[0] = m_originalData.at(i);
				data->d_byte_array->replace(h*128+w, 1, &color[0], 1);
				i++;
			}
		}
	}

}
void CommandRailMap::redoCommand()
{
	m_originalData.clear();
	doCommand();
}
