/******************************************************************************
 * consoleman.h - Manage the app messages
 *
 * yamce - Yet Another Minecraft Editor
 * Copyright(c) 2019 - Dick Balaska, and BuckoSoft.
 *
 * yamce is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * yamce is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *****************************************************************************/
#ifndef CONSOLEMAN_H
#define CONSOLEMAN_H

#include <QObject>

class ConsoleDialog;
class MainWindow;
class WindowSettings;
class World;

class ConsoleMsg
{
public:
	QtMsgType	m_type;
	QString		m_string;
};

class ConsoleMan : public QObject
{
	Q_OBJECT
public:
	ConsoleMan(MainWindow* mainWindow);
	void	setWorld(World* world);

	void	consoleMessage(QtMsgType type, const QMessageLogContext &context, const QString &msg);

	void	updateConsoleDialog();
	void	resetConsoleDialog();
	void	setupConsoleDialog(WindowSettings* settings);

	bool	getShowDebug();
	void	setShowDebug(bool showDebug);

public slots:
	void	onConsoleDialog(bool);

private slots:
	void	onConsoleDialogClosed();

private:

	MainWindow*			m_mainWindow;
	World*				m_world;
	ConsoleDialog*		m_consoleDialog;
	WindowSettings*		m_consoleDialogSettings;
	QList<ConsoleMsg*>	m_messageQueue;
	bool				m_showDebug;
};

inline bool ConsoleMan::getShowDebug() { return(m_showDebug); }
inline void	ConsoleMan::setShowDebug(bool showDebug) { m_showDebug = showDebug; }

#endif // CONSOLEMAN_H
