#ifndef MAPITEM_H
#define MAPITEM_H

#include <QPoint>
#include <QList>

#include "nbtfile.h"

class QPixmap;

class MapBanner
{
public:
	QPoint m_pos;
	QString m_color;
};

class MapFile : public NBTFile
{
private:
	typedef	NBTFile super;

public:
	MapFile();
	virtual ~MapFile();

	virtual bool load(ErrorMsgReceiver* receiver = nullptr);

	bool			parseMap(ErrorMsgReceiver* receiver = nullptr);

	bool			hasMapImage();
	const QPixmap*	getPixmap();
	void			setPos(const QPoint& point);
	const QPoint	getPos();
	bool			isExplorerMap();
	bool			isImageDisplayed();
	void			setImageDisplayed(bool b);

private:
	QPixmap*		m_pixmap;
	QPoint			m_pos;			// center of the map in world coordinates
	int				m_scale;
	int				m_dimension;
	bool			m_explorerMap;
	// Don't draw this image until we determine its draw rect.
	// Due to multithreaded reads, we can see there's an image before we have finished loading it
	bool			m_imageProcessed;
	bool			m_railsApplied;	// used for stats gathering and then again to see if this map needs saving.

	bool			m_imageDisplayed;	// Did the user hide/mute this map?
	QList<MapBanner*>	m_mapBanners;

	friend class MapMapWidget;
	friend class RailMan;
	friend class RailMapThread;
};

const QList<QColor>& getMapColorMap();

inline 	bool MapFile::hasMapImage() { return(m_pixmap != nullptr); }
inline const QPixmap* MapFile::getPixmap() { return(m_pixmap); }
inline void MapFile::setPos(const QPoint &point) { m_pos = point; }
inline const QPoint MapFile::getPos() { return(m_pos); }
inline bool MapFile::isExplorerMap() { return(m_explorerMap); }
inline bool MapFile::isImageDisplayed() { return(m_imageDisplayed); }
inline void MapFile::setImageDisplayed(bool b) { m_imageDisplayed = b; }
#endif // MAPITEM_H
