/******************************************************************************
 * mapsettingsdialog.cpp - Dialog to control the map displays
 *
 * yamce - Yet Another Minecraft Editor
 * Copyright(c) 2018 - Dick Balaska, and BuckoSoft.
 *
 * yamce is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * yamce is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *****************************************************************************/
#include <QDebug>
#include <QCheckBox>
#include <QPushButton>
#include <QSpinBox>
#include <QLineEdit>
#include <QTextEdit>
#include <QGroupBox>
#include <QGridLayout>
#include <QLabel>
#include <QTimer>

#include "mapwidget.h"
#include "world.h"
#include "mapsettingsdialog.h"
#include "mainwindow.h"
//#include "railman.h"


MapSettingsDialog::MapSettingsDialog(MapWidget* parent, MainWindow* mainWindow)
	: QDialog(parent),
	  m_mapWidget(parent),
	  m_mainWindow(mainWindow),
	  m_world(nullptr)
{
	const int numCols=3;
	const int row=0;
	const int rowSpan=1;
	enum {
		mainCol,
		widgCol,
		statCol
	};

	//QVBoxLayout* mainLayout = new QVBoxLayout();
	QGridLayout* layout = new QGridLayout();
	layout->setColumnMinimumWidth(0, 20);
	setWindowTitle("yamce - Map Settings");
	//QVBoxLayout*	layout = new QVBoxLayout();

	m_drawGridLines = new QCheckBox("Draw Grid Lines");
	connect(m_drawGridLines, SIGNAL(toggled(bool)), this, SLOT(drawGridLinesToggled(bool)));
	layout->addWidget(m_drawGridLines,			row+0, mainCol, rowSpan, numCols);

	m_gridWidthSpinBox = new QSpinBox();
	m_gridWidthSpinBox->setValue(1);
	m_gridWidthSpinBox->setMinimum(1);
	connect(m_gridWidthSpinBox, SIGNAL(valueChanged(int)), this, SLOT(gridWidthChanged(int)));
	layout->addWidget(m_gridWidthSpinBox,		row+1, widgCol, rowSpan, numCols-1);

	m_expandForGrid = new QCheckBox("Expand map for gridlines");
	connect(m_expandForGrid, SIGNAL(toggled(bool)), this, SLOT(expandForGridChanged(bool)));
	layout->addWidget(m_expandForGrid,			row+2, widgCol, rowSpan, numCols-1);


	//mainLayout->addLayout(layout);
	//mainLayout->addStretch(100);
	//this->setLayout(mainLayout);
	this->setLayout(layout);
	maybeEnableOptions();
}

void MapSettingsDialog::closeEvent(QCloseEvent* event)
{
//	m_isClosing = true;
	super::closeEvent(event);
}


void MapSettingsDialog::setWorld(World* world)
{
	m_world = world;
	m_yamceDoc = &world->m_yamceDoc;
	MapSettings& ms = m_yamceDoc->m_mapSettings;
	m_drawGridLines->setChecked(ms.m_drawGridLines);
	m_gridWidthSpinBox->setValue(ms.m_gridWidth);
	m_expandForGrid->setChecked(ms.m_expandForGrid);
	maybeEnableOptions();
}

void MapSettingsDialog::maybeEnableOptions()
{
	bool b = m_drawGridLines->isChecked();
	m_gridWidthSpinBox->setEnabled(b);
	m_expandForGrid->setEnabled(b);
}

void MapSettingsDialog::drawGridLinesToggled(bool b)
{
	m_yamceDoc->m_mapSettings.m_drawGridLines = b;
	maybeEnableOptions();
	emit (mapSettingsChanged());
}

void MapSettingsDialog::gridWidthChanged(int value)
{
	m_yamceDoc->m_mapSettings.m_gridWidth = value;
	emit (mapSettingsChanged());
}

void MapSettingsDialog::expandForGridChanged(bool b)
{
	m_yamceDoc->m_mapSettings.m_expandForGrid = b;
	emit (mapSettingsChanged());
}

