/******************************************************************************
 * nbtfile.h - Read and write and NBT file
 *
 * yamce - Yet Another Minecraft Editor
 * Copyright(c) 2018 - Dick Balaska, and BuckoSoft.
 *
 * yamce is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * yamce is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *****************************************************************************/
#ifndef NBTFILE_H
#define NBTFILE_H

#include <QString>
#include <QDateTime>

#include "nbtobject.h"

class QWidget;
class NList;
class ErrorMsgReceiver;
class MainWindow;

#define errorMessage(__s) { \
	if (m_errorMsgReceiver) \
		qCritical() << __s; \
	m_errorMsgReceiver = nullptr; }


class NBTFile
{
public:
	NBTFile();
	virtual ~NBTFile();


	void init(const QString& dirname, const QString& filename, bool compressed = true);
	virtual bool load(ErrorMsgReceiver* receiver);
	virtual bool save(ErrorMsgReceiver* receiver);
	virtual void unload();

	const QString& getDirName() { return(m_dirName); }
	const QString& getFileName() { return(m_fileName); }
	bool	isDirty() { return(m_dirty); }
	void	setDirty(bool b = true) { m_dirty = b; }
	bool	isLoaded() { return(m_loaded); }

	NBTObject*	getObject() { return(m_object); }
	NBTObject*	findObject(const QString& name);

	void		setLastUse();		// Update the timestamp that we used this file

protected:
	NBTObject*	parse(NBTObject* obj);
	qint8		fetchByte();
	qint16		fetchShort();
	qint32		fetchInt();
	qint64		fetchLong();
	QString		fetchString();
	float		fetchFloat();
	double		fetchDouble();

	void		serializeObject(const NBTObject* obj, bool inList = false);
	void		putByte(quint8 b);
	void		putShort(qint16 w);
	void		putInt(qint32 i);
	void		putLong(qint64 l);
	void		putFloat(float f);
	void		putDouble(double d);
	void		putString(const QString& s);

	QString		m_dirName;
	QString		m_fileName;
	bool		m_compressed;
	int			m_parseOffset;
	NBTObject*	m_object;
	QByteArray	m_data;
	bool		m_loaded;
	bool		m_dirty;
	QDateTime	m_lastUse;
	ErrorMsgReceiver*	m_errorMsgReceiver;

private:
	bool		parseList(NList* nlist);

	bool		m_fileManReading;		// Fileman is actively reading this file

	friend class World;
	friend class FileMan;
};

inline void NBTFile::setLastUse() { m_lastUse = QDateTime::currentDateTimeUtc(); }
inline NBTObject* NBTFile::findObject(const QString &name) { return(m_object->findChild(name)); }
#endif // NBTFILE_H
