/******************************************************************************
 * preferencesdialog.h - Dialog to control various settings
 *
 * yamce - Yet Another Minecraft Editor
 * Copyright(c) 2019 - Dick Balaska, and BuckoSoft.
 *
 * yamce is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * yamce is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *****************************************************************************/

#ifndef PREFERENCESDIALOG_H
#define PREFERENCESDIALOG_H

#include <QDialog>

class QCheckBox;
class QLineEdit;
class QSpinBox;
class MainWindow;
class SystemTab;
class World;
class WorldTab;

class PreferencesDialog : public QDialog
{
	Q_OBJECT

public:
	explicit	PreferencesDialog(MainWindow* mainWindow);
	void		setWorld(World* world);
	World*		getWorld();
	MainWindow*	getMainWindow();


private:
	MainWindow*	m_mainWindow;
	World*		m_world;
	SystemTab*	m_systemTab;
	WorldTab*	m_worldTab;
};

inline World*	PreferencesDialog::getWorld() { return(m_world); }
inline MainWindow*	PreferencesDialog::getMainWindow() { return(m_mainWindow); }

class SystemTab : public QWidget
{
	Q_OBJECT
public:
	explicit SystemTab(PreferencesDialog* parent);

private slots:
	void	onHelpDirectoryChanged();
	void	onHelpDirectoryBrowse();
	void	onShowDebugToggled(int newState);

private:
	MainWindow*	m_mainWindow;
	QLineEdit*	m_helpDirLineEdit;
	QCheckBox*	m_showDebug;
};

class WorldTab : public QWidget
{
	Q_OBJECT
public:
	explicit WorldTab(PreferencesDialog* parent);
	void	setWorld(World* world);

private slots:
	void	onMapWallEdited();
	void	onMapWallBrowse(bool);
	void	onMapsToWatchChanged(int);

private:
	World*		m_world;
	QLineEdit*	m_mapWallLineEdit;
	QSpinBox*	m_mapsToWatch;
};

#endif // PREFERENCESDIALOG_H
