/******************************************************************************
 * railmandialog.h - Dialog to control RailMan
 *
 * yamce - Yet Another Minecraft Editor
 * Copyright(c) 2019 - Dick Balaska, and BuckoSoft.
 *
 * yamce is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * yamce is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *****************************************************************************/
#ifndef RAILMANDIALOG_H
#define RAILMANDIALOG_H

#include <QDialog>
#include <QLineEdit>
#include <QFocusEvent>

class QCheckBox;
class QSpinBox;
class QLineEdit;
class QTextEdit;
class QLabel;
class QGridLayout;

class MapWidget;
class MainWindow;
class MapColorDialog;
class NBTFile;
class RailMan;
class RegionFile;
class World;
class XYZ;
class YamceDoc;

class MSDLineEdit : public QLineEdit
{
	Q_OBJECT
protected:
	virtual void focusOutEvent(QFocusEvent *e) override	{
		e->accept();
	}

};

class RailStartEditWithStatus
{
public:
	RailStartEditWithStatus() :
		m_railRootNeedsValidating(false),
		m_railRootValidated(false),
		m_railRootRegionFile(nullptr),
		m_xyz(nullptr) {}

	MSDLineEdit*	m_lineEdit;
	QLabel*			m_status;
	bool			m_railRootNeedsValidating;		// true if we should read the file and check
	bool			m_railRootValidated;			// true if rail exists at this location
	RegionFile*		m_railRootRegionFile;
	XYZ*			m_xyz;
};

class RailManDialog : public QDialog
{
private:
	typedef QDialog super;
	Q_OBJECT

public:
	RailManDialog(MapWidget* parent, MainWindow* mainWindow);
	virtual	~RailManDialog();

	void	setWorld(World* world);
	void	closing();

signals:
	void	mapSettingsChanged();
	void	processRailsChecked();


public slots:
	void	fileLoaded(NBTFile* file);
	void	onRailManDone();


private slots:
	void	drawRailLinesToggled(bool b);
	void	railStartEditingFinished();
	void	processRailLinesToggled(bool);
	void	onMinecraftMapsToggled(bool);
	void	onTimerTick();
	void	onColorButtonClicked();
	void	onMapColorDialogClosed();
	void	onMapColorChanged(int mapColor);

private:
	void		maybeEnableOptions();
	void		updateRailRootStatus();
	RailStartEditWithStatus*	findEditStatus(QLineEdit* lineEdit);
	void		setupRailStartEdits();
	void		setRailManIsRunning(bool running = true);
	void		setColorButtonIcon();

	MainWindow*		m_mainWindow;
	RailMan*		m_railMan;
	QCheckBox*		m_drawRailLines;
	QTextEdit*		m_railStatsEdit;
	QPushButton*	m_colorButton;
	QPushButton*	m_processRailLines;
	QPushButton*	m_updateMinecraftMaps;
	MapColorDialog*	m_mapColorDialog;

	World*			m_world;
	YamceDoc*		m_yamceDoc;
	bool			m_railManIsRunning;
	QTimer*			m_timer;
	QGridLayout*	m_railStartEditLayout;
	QList<RailStartEditWithStatus*>	m_railStartEditList;

	// we need to track this because the stupid railStatsEdit signals "done" on destruction
	bool			m_isClosing;
};

inline void RailManDialog::closing() { m_isClosing = true; }
inline void RailManDialog::setRailManIsRunning(bool running) { m_railManIsRunning = running; }
#endif // RAILMANDIALOG_H
