/******************************************************************************
 * regionfile.h - Read and write a Minecraft region file
 *
 * yamce - Yet Another Minecraft Editor
 * Copyright(c) 2018 - Dick Balaska, and BuckoSoft.
 *
 * yamce is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * yamce is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *****************************************************************************/
#ifndef REGIONFILE_H
#define REGIONFILE_H

#include "xyz.h"
#include "nbtfile.h"


const int CPF = 1024;		// chunks per file

class RegionFile : public NBTFile
{
private:
	typedef NBTFile super;

public:
	RegionFile();
	virtual ~RegionFile();

	virtual bool load(ErrorMsgReceiver* receiver = nullptr);
	virtual bool save(ErrorMsgReceiver* receiver = nullptr);

	/// Unload the data from this file to free up space. Keep the initial skeleton data intact.
	virtual void	unload();

	NBTObject*	getBlockPalette(const XYZ& xyz);

private:
	quint32*	m_locations;
	quint32*	m_timestamps;
	NBTObject**	m_chunks;

	int			m_x;			// x/z coordinate for maps and regions
	int			m_z;

	friend class RegionMapWidget;
	friend class World;
	friend class TreeModel;
	friend class ResourceDock;
	friend class RailMan;
};

#endif // REGIONFILE_H
