/******************************************************************************
 * resourcedock.h - The left widget in the MainWindow
 *
 * yamce - Yet Another Minecraft Editor
 * Copyright(c) 2018 - Dick Balaska, and BuckoSoft.
 *
 * yamce is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * yamce is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *****************************************************************************/
#ifndef RESOURCEDOCK_H
#define RESOURCEDOCK_H

#include <QDockWidget>
#include <QTreeView>

#include "world.h"
#include "mainwindow.h"

class ResourceDropDown;
class TreeItem;
class TreeModel;
class TreeView;

///////////////////////////////////////////////////////////////////////////////////////////////////
/// \brief Manage the TreeView and Group selector that live on the left side of the window
///
class ResourceDock : public QDockWidget
{
	Q_OBJECT
	friend class TreeView;
public:
	explicit ResourceDock(MainWindow* parent, Qt::WindowFlags flags = 0);
	virtual ~ResourceDock();

	void		setWorld(World* world);

	void		closing();
	void		captureWidgets();

	bool		setGroupDropdownValue(int v);
	void		setupColumnWidths();
	const NBTFile*	getCurrentFile();
	NBTFile*	getFile(TreeItem* treeItem);
	TreeItem*	getTreeItem(NBTFile* file);

	int			getCurrentGroupIndex();
	Group*		getCurrentGroup();

	TreeView*	getTreeView();
	//TreeModel*	getTreeModel();
	void		addMap(MapFile* file);

public slots:
	void	explorerTypeChanged(int newGroup);
	/// Region files operate on one region at a time.  Replace the TreeModel
	void	setRegionFile(NBTFile* file);
	void	fileLoaded(NBTFile* file);
	void	setMapFile(MapFile* mapFile);
	void	onDataEdited(const QModelIndex& index, const QVariant data);

private slots:
	void	nodeDoubleClicked(const QModelIndex& index);

protected:
	virtual	void closeEvent(QCloseEvent* event);


private:
	//void				regionFileLoaded(RegionFile* file);

	MainWindow*			m_mainWindow;
	ResourceDropDown*	m_dropdown;
	TreeView*			m_treeView;
	//TreeModel*			m_treeModel;
	World*				m_world;
	int					m_currentGroup;
	NBTFile*			m_currentFile;
	Group*				m_group;
};

inline const NBTFile* ResourceDock::getCurrentFile() { return(m_currentFile); }
inline TreeView* ResourceDock::getTreeView() { return(m_treeView); }
//inline TreeModel* ResourceDock::getTreeModel() { return(m_treeModel); }
inline int ResourceDock::getCurrentGroupIndex() { return(m_currentGroup); }
inline Group* ResourceDock::getCurrentGroup() { return(m_group); }

#endif // RESOURCEDOCK_H
