/******************************************************************************
 * treeitem.cpp - One entry in the Resource tree
 *
 * yamce - Yet Another Minecraft Editor
 * Copyright(c) 2018 - Dick Balaska, and BuckoSoft.
 *
 * yamce is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * yamce is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *****************************************************************************/
#include "treeitem.h"
#include "nbtobject.h"

TreeItem::TreeItem(const QList<QVariant>& data, NBTObject* object, TreeItem* parent)
{
	m_parentItem = parent;
	m_itemData = data;
	m_nbtObject = object;
}

TreeItem::~TreeItem()
{
	qDeleteAll(m_childItems);
}

void TreeItem::appendChild(TreeItem* item)
{
	m_childItems.append(item);
}

void TreeItem::replaceChild(TreeItem* child, int row)
{
	TreeItem* old = m_childItems[row];
	m_childItems[row] = child;
	delete old;
}

TreeItem* TreeItem::removeChild(int row)
{
	TreeItem* child = m_childItems[row];
	m_childItems.removeAt(row);
	return(child);
}

void TreeItem::insertChild(TreeItem *child, int row)
{
	m_childItems.insert(row, child);
}

TreeItem* TreeItem::child(int row)
{
	return m_childItems.value(row);
}

int TreeItem::childCount() const
{
	return m_childItems.count();
}

int TreeItem::row() const
{
	if (m_parentItem)
		return m_parentItem->m_childItems.indexOf(const_cast<TreeItem*>(this));
	return 0;
}

int TreeItem::columnCount() const
{
	return m_itemData.count();
}

QVariant TreeItem::data(int column) const
{
	return m_itemData.value(column);
}

void TreeItem::setDataValue(QVariant value, int column)
{
	m_itemData[column] = value;
}

TreeItem* TreeItem::parentItem()
{
	return m_parentItem;
}

QString	TreeItem::getPath()
{
	QString	output;
	TreeItem* parent = this->parentItem();
	NBTObject* obj;
	TreeItem*	ti = this;
	QString s;
	while (parent) {
//		if (!parent->parentItem())
//			break;			// Don't include the hidden root in our path
		obj = ti->getObject();
		if (!obj)
			break;
		NBTObject* pobj = parent->getObject();
		s = ti->m_itemData.at(0).toString();
		if (s.isEmpty()) {
			if (pobj && pobj->type == TAG_List) {
				int i = pobj->getChildIndex(obj);
				if (i == -1 && s.isEmpty()) {
					output = QString("%1:*").arg(output);
				} else {
					output = QString("[%1]%2").arg(i).arg(output);
				}
			}
		} else {
			if (obj->type == TAG_List)
				output = QString("%1%2").arg(s, output);
			else if (!output.isEmpty())
				output = QString("%1:%2").arg(s, output);
			else {
				output = s;
			}
		}
		ti = parent;
		parent = parent->parentItem();
	}
	return(output);
}

