/******************************************************************************
 * treeitem.h - One entry in the Resource tree
 *
 * yamce - Yet Another Minecraft Editor
 * Copyright(c) 2018 - Dick Balaska, and BuckoSoft.
 *
 * yamce is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * yamce is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *****************************************************************************/
#ifndef TREEITEM_H
#define TREEITEM_H

#include <QVariant>
#include <QList>

class NBTObject;
class NBTFile;

/** A classic Tree node.
 */
class TreeItem
{
public:
	explicit TreeItem(const QList<QVariant> &data, NBTObject* object, TreeItem *parentItem = 0);
	~TreeItem();

	void		appendChild(TreeItem* child);
	void		replaceChild(TreeItem* child, int row);
	TreeItem*	removeChild(int row);
	void		insertChild(TreeItem* child, int row);

	/// Get the path to this object. i.e. "map_0.dat:data:banners[1]:Pos"
	QString		getPath();

	TreeItem*	child(int row);
	int			childCount() const;
	int			columnCount() const;
	QVariant	data(int column) const;
	void		setDataValue(QVariant value, int column = 2);
	int			row() const;
	TreeItem*	parentItem();

	const QList<TreeItem*>&	getChildren();

	NBTObject*	getObject();
	void		setObject(NBTObject* object);

	const QList<QVariant> getItemData();
	QString		getName();

private:
	QList<TreeItem*> m_childItems;
	QList<QVariant> m_itemData;
	TreeItem*		m_parentItem;
	NBTObject*		m_nbtObject;	// pointer to original object.  We don't own this object.

	friend class TreeView;
};


inline NBTObject*	TreeItem::getObject() { return(m_nbtObject); }
inline void			TreeItem::setObject(NBTObject *object) { m_nbtObject = object; }
inline const QList<QVariant> TreeItem::getItemData() { return(m_itemData); }
inline const QList<TreeItem*>&	TreeItem::getChildren() {return(m_childItems); }
inline QString TreeItem::getName() {
	QString s = m_itemData.at(0).toString();
	if (s.endsWith("*"))
		return(s.left(s.length()-1));
	else
		return(s);
}

#endif // TREEITEM_H
