/******************************************************************************
 * treemodel.h - Extend QAbstractItemModel to support our tree
 *
 * yamce - Yet Another Minecraft Editor
 * Copyright(c) 2018 - Dick Balaska, and BuckoSoft.
 *
 * yamce is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * yamce is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *****************************************************************************/
#ifndef TREEMODEL_H
#define TREEMODEL_H

#include <QAbstractItemModel>

//class World;
class TreeItem;
class NBTObject;
class RegionFile;

enum {
	tmName,
	tmType,
	tmData
};

class TreeModel : public QAbstractItemModel
{
	Q_OBJECT

public:
	explicit TreeModel(QObject* parent);
	~TreeModel();

	QVariant		data(const QModelIndex& index, int role = Qt::DisplayRole) const override;
	bool			setData(const QModelIndex &index, const QVariant &value, int role) override;
	Qt::ItemFlags	flags(const QModelIndex& index) const override;
	QVariant		headerData(int section, Qt::Orientation orientation,
							   int role = Qt::DisplayRole) const override;
	QModelIndex		index(int row, int column,
						  const QModelIndex& parent = QModelIndex()) const override;
	QModelIndex		parent(const QModelIndex& index) const override;
	int				rowCount(const QModelIndex& parent = QModelIndex()) const override;
	int				columnCount(const QModelIndex& parent = QModelIndex()) const override;

	bool		isEmpty();
	TreeItem*	addNBTModelData(NBTObject* root, const QString& label, bool noInsert = false);
	TreeItem*	addRegionModelData(RegionFile* file, bool noInsert = false);

	TreeItem*	getTreeItem(const QModelIndex& index);
	TreeItem*	getTreeItem(const QString& path, TreeItem* node);
	QModelIndex	getIndex(const QString& path);
	TreeItem*	getRootNode();

	void		replaceRow(QModelIndex& parent, int row, TreeItem* treeItem);
	TreeItem*	removeRow(QModelIndex& parent, int row);
	void		insertRow(QModelIndex& parent, int row, TreeItem* treeItem);

signals:
	void		dataEdited(const QModelIndex& index, const QVariant data);

private:
	void setupModelData(NBTObject* root, TreeItem* parent);
	void setupTree(NBTObject* obj, TreeItem* parent);


	TreeItem*	m_rootItem;
	QObject*	m_parent;
};

inline TreeItem* TreeModel::getRootNode() { return(m_rootItem); }

#endif // TREEMODEL_H
