/******************************************************************************
 * world.h - Describe one Minecraft world
 *
 * yamce - Yet Another Minecraft Editor
 * Copyright(c) 2018 - Dick Balaska, and BuckoSoft.
 *
 * yamce is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * yamce is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *****************************************************************************/
#ifndef WORLD_H
#define WORLD_H

#include <QObject>

#include "xyz.h"
#include "yamcedoc.h"

class ErrorMsgReceiver;
class MainWindow;
class NBTFile;
class RegionFile;
class PlayerFile;
class TreeModel;

enum {
	wgMain,
	wgPlayers,
	wgMaps,
	wgVillages,
	wgOverworldRegions,
	wgNetherRegions,
	wgEndRegions,

	wgMAX
};

//typedef QList<NBTFile*> WGroup;
extern	QStringList mainEntries;
class Group
{
public:
	Group();
	virtual ~Group();
	void	append(NBTFile* file);
	QList<NBTFile*>&	getFiles();
	TreeModel*			getTreeModel();
	void				setTreeModel(TreeModel* treeModel);

private:
	QList<NBTFile*> m_files;
	TreeModel*		m_tree;

	friend class World;
};
inline void				Group::append(NBTFile* file) { m_files.append(file); }
inline QList<NBTFile*>& Group::getFiles() { return(m_files); }
inline TreeModel*		Group::getTreeModel() { return(m_tree); }
inline void				Group::setTreeModel(TreeModel* treeModel) { m_tree = treeModel; }

class World : QObject
{
	Q_OBJECT
	friend class TreeModel;
	friend class RegionMapWidget;

public:
	World(MainWindow *mainWindow);
	~World();

	bool	load(const QString& dirname, ErrorMsgReceiver* receiver = nullptr);
	bool	save(ErrorMsgReceiver* receiver = nullptr);

	QList<Group*>&	getGroups() { return(m_groups); }
	Group*			getGroup(int which) { return(m_groups.at(which)); }
	const QString&	getName() { return(m_name); }
	const QString&	getBaseDirectory();
	QString			getWorldDirectory();
	QString			getMapWallFileName();

	Group*			findGroup(NBTFile* file);
	RegionFile*		findRegion(int x, int z);
	RegionFile*		findRegion(const QPoint& xz);
	RegionFile*		findRegion(const XYZ& xyz);
	PlayerFile*		findPlayer(const QString& fileName);
	MapFile*		findMap(const QString& fileName);

	YamceDoc		m_yamceDoc;
private:
	MainWindow*		m_mainWindow;
	QString			m_directory;
	bool			m_serverLayout;
	QString			m_name;
	//NBTFile*		m_leveldat;
	//QList<NBTFile*>	m_files;
	QList<Group*>	m_groups;
	int				m_minRegionX;
	int				m_minRegionZ;
	int				m_maxRegionX;
	int				m_maxRegionZ;
};
inline const QString& World::getBaseDirectory() { return(m_directory); }
inline QString World::getWorldDirectory() { if (!m_serverLayout) return(m_directory); return(QString("%1/world").arg(m_directory)); }
inline RegionFile* World::findRegion(const QPoint& xz) { return(findRegion(xz.x(), xz.y())); }
inline RegionFile* World::findRegion(const XYZ& xyz) { return(findRegion(xyz.x, xyz.z)); }

inline QPoint blockToRegion(QPoint p) { return(QPoint(p.x()>>9, p.y()>>9)); }
inline QPoint blockToRegion(int x, int z) { return(QPoint(x >> 9, z >> 9)); }
inline QPoint blockToRegion(const XYZ& xyz) { return(QPoint(xyz.x >> 9, xyz.z >> 9)); }

#endif // WORLD_H
