/******************************************************************************
 * xyz.h - a 3D point
 *
 * yamce - Yet Another Minecraft Editor
 * Copyright(c) 2019 - Dick Balaska, and BuckoSoft.
 *
 * yamce is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * yamce is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *****************************************************************************/
#ifndef XYZ_H
#define XYZ_H

#include <QString>
#include <QPoint>
#include <QRect>

class XYZ
{
public:
	XYZ() { x=y=z=0;  }
	XYZ(int _x, int _y, int _z) { x=_x; y=_y; z=_z; }
	XYZ& operator+=(const XYZ& other);
	QString toString(bool useSpaces = false);
	XYZ&	fromString(const QString& in, bool* usesSpaces = nullptr, bool* success = nullptr);
	bool	inside(const QRect& rect);
	QPoint	toQPoint();

	int	x;
	int y;
	int z;
};

inline XYZ& XYZ::operator+=(const XYZ& other) {this->x += other.x; this->y += other.y; this->z += other.z; return(*this); }

inline bool operator==(const XYZ& a, const XYZ& b) { return(a.x == b.x && a.y == b.y && a.z == b.z); }

QDebug& operator<<(QDebug & stream, const XYZ& xyz);
QDataStream& operator<<(QDataStream& stream, const XYZ& xyz);
QDataStream& operator>>(QDataStream& stream,       XYZ& xyz);

inline QPoint blockToChunk(const XYZ& xyz) { return(QPoint(xyz.x >> 4, xyz.z >> 4)); }

inline bool XYZ::inside(const QRect &rect)
{
	if (x >= rect.left() && x <= rect.right() && z >= rect.top() && z <= rect.bottom())
		return(true);
	return(false);
}
inline QPoint XYZ::toQPoint() { return(QPoint(x,z)); }

#endif // XYZ_H
