/******************************************************************************
 * yamcedoc.h - Manage the yamce accoutrements to a World
 *
 * yamce - Yet Another Minecraft Editor
 * Copyright(c) 2018 - Dick Balaska, and BuckoSoft.
 *
 * yamce is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * yamce is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *****************************************************************************/
#ifndef YAMCEDOC_H
#define YAMCEDOC_H

#include <QList>

#include "xyz.h"

class QDir;
class MapFile;
class MainWindow;
class World;


class MapSummary
{
public:
	QPoint	m_pos;
	QList<MapFile*>	m_files;
};


class MapSettings
{
public:
	MapSettings()
		: m_gridWidth(1), m_drawGridLines(true),
		  m_expandForGrid(false), m_railPointSpaces(true),
		  m_displayMapImages(true), m_mapZoomDisplayed(0) {}
	~MapSettings();
	//MapSettings& operator=(const MapSettings& other);
	QStringList			encodeRailStartPoints();
	void				decodeRailStartPoints(const QStringList& in);

	int			m_gridWidth;
	bool		m_drawGridLines;
	bool		m_expandForGrid;
	bool		m_drawRailLines;
	QList<XYZ*>	m_railStartPoints;
	bool		m_railPointSpaces;	// use spaces instead of commas

	bool		m_displayMapImages;
	int			m_mapZoomDisplayed;


};

class WindowSettings
{
public:
	WindowSettings() { m_shown = false; }
	QRect		m_pos;
	bool		m_shown;
};

class YamceDoc
{
public:
	YamceDoc();

	bool load(MainWindow* mainWindow, World* world);
	bool save(MainWindow* mainWindow, World* world);
	QDir getYamceDir(World* world, bool createDir = false);

	MapSettings		m_mapSettings;
	WindowSettings	m_mapSettingsDialogSettings;
	WindowSettings	m_railManDialogSettings;
	WindowSettings	m_consoleDialogSettings;
	WindowSettings	m_prefsDialogSettings;

	QList<QVariant>	m_resourceColumnWidths;
	int				m_currentResourceGroup;
	double			m_currentZoom;
	QString			m_exportPngDir;
	int				m_maxMapsWatched;
	/// map color index
	int				m_railColor;

};

#endif // YAMCEDOC_H
